@@ -7,30 +7,46 @@ from pathlib import Path
 
 import pytest
 
+import shade.v1.api
 from shade import ShadeLocal
+from typing import List
+from shade.v1.models import Jobs
 
 
-def wait_for_no_jobs(status_response: dict):
-    if status_response['state'] != 'IDLE':
-        return False
-    for job in status_response['jobs']:
-        if job['running']:
-            return False
-
-    return True
+# Wooo these can all be fixtures
+def wait_for_assets(backend: ShadeLocal, paths: List[Path]):
+    """
+    Wait for assets to be indexed
+    """
+    while True:
+        try:
+            # Make sure they're all there
+            [backend.assets.get_asset_by_path(asset) for asset in paths]
+            break
+        except shade.v1.api.APIException:
+            time.sleep(1)
 
 
-def wait_for_indexing(backend, count=0):
-    if count >= 40:
-        return
+def wait_for_jobs(backend: ShadeLocal, jobs: List[Jobs], paths: List[Path] = None):
+    """
+    Get all assets and check to make sure that each one of the jobs on the assets is either completed or failed
+    """
+    wait_for_assets(backend, paths)
 
-    status_response = backend.indexing.status()
-    if wait_for_no_jobs(status_response):
-        time.sleep(0.25)
-        wait_for_indexing(backend, count + 1)
-    else:
-        time.sleep(0.25)
-        wait_for_indexing(backend, count)
+    job_keys = [job.value for job in jobs]
+
+    def __asset_has_jobs(asset):
+        for job in job_keys:
+            if getattr(asset, job) not in ('COMPLETED', 'FAILED'):
+                return False
+        return True
+
+    while True:
+        assets = [backend.assets.get_asset_by_path(asset) for asset in paths]
+        if not all([__asset_has_jobs(asset) for asset in assets]):
+            time.sleep(1)
+        else:
+            break
 
 
 # @pytest.mark.parametrize('asset_name', [
@@ -113,7 +129,7 @@ def wait_for_indexing(backend, count=0):
 @pytest.mark.parametrize('demo_file_names', [
     ([
         'video/coverr-berlin-underground-train-7268-original.mp4',
-        'video/braw-r3d/A002_C305_0523UB_001.R3D',
+        # 'video/braw-r3d/A002_C305_0523UB_001.R3D',
         # TODO doesn't seem like braw is working
         # 'video/In-The-Hand-Original.braw',
 
@@ -146,7 +162,12 @@ def test_visual_assets(
     backend.models.enable_model('braw')
     root_id = backend.roots.add_new_root(tmp_path)
 
-    wait_for_indexing(backend)
+    wait_for_jobs(backend, [
+        Jobs.PREVIEWS,
+        Jobs.CORE,
+        Jobs.COLOR_PALETTE,
+        Jobs.METADATA
+    ], demo_file_paths)
 
     try:
         for asset in demo_file_paths:
@@ -158,6 +179,7 @@ def test_visual_assets(
             assert asset_.type
             assert asset_.signature
             assert asset_.ai_indexed
+            assert len(asset_.palette) > 0
 
             assert len(asset_.preview_images) > 0
 
@@ -184,7 +206,10 @@ def test_audio_assets(
 
     root_id = backend.roots.add_new_root(tmp_path)
 
-    wait_for_indexing(backend)
+    wait_for_jobs(backend, [
+        Jobs.METADATA,
+        Jobs.AUDIO
+    ], demo_file_paths)
 
     try:
         for asset in demo_file_paths:
@@ -218,7 +243,10 @@ def test_text_assets(
 
     root_id = backend.roots.add_new_root(tmp_path)
 
-    wait_for_indexing(backend)
+    wait_for_jobs(backend, [
+        Jobs.METADATA,
+        Jobs.TEXT
+    ], demo_file_paths)
 
     try:
         for asset in demo_file_paths:
@@ -246,7 +274,10 @@ def test_facial_recognition(
 
     root_id = backend.roots.add_new_root(tmp_path)
 
-    wait_for_indexing(backend)
+    wait_for_jobs(backend, [
+        Jobs.METADATA,
+        Jobs.FACIAL_RECOGNITION
+    ], demo_file_paths)
 
     try:
         for asset in demo_file_paths:
